<?php

namespace Drupal\media_pdf_thumbnail\Manager;

use Drupal\Core\Entity\Entity;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\File\FileSystem;

/**
 * Class MediaPdfThumbnailImageManager.
 *
 * @package Drupal\media_pdf_thumbnail\Manager
 */
class MediaPdfThumbnailImageManager {

  /**
   * MediaPdfThumbnailManager.
   *
   * @var \Drupal\media_pdf_thumbnail\Manager\MediaPdfThumbnailImagickManager
   */
  protected $mediaPdfThumbnailImagickManager;

  /**
   * EntityTypeManager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManager
   */
  protected $entityTypeManager;

  /**
   * FileSystem.
   *
   * @var \Drupal\Core\File\FileSystem
   */
  protected $fileSystem;

  /**
   * MediaPdfThumbnailManager constructor.
   *
   * @param \Drupal\media_pdf_thumbnail\Manager\MediaPdfThumbnailImagickManager $mediaPdfThumbnailImagickManager
   *   $mediaPdfThumbnailImagickManager.
   * @param \Drupal\Core\Entity\EntityTypeManager $entityTypeManager
   *   EntityTypeManager.
   * @param \Drupal\Core\File\FileSystem $fileSystem
   *   FileSystem.
   */
  public function __construct(
    MediaPdfThumbnailImagickManager $mediaPdfThumbnailImagickManager,
    EntityTypeManager $entityTypeManager,
    FileSystem $fileSystem
  ) {
    $this->mediaPdfThumbnailImagickManager = $mediaPdfThumbnailImagickManager;
    $this->entityTypeManager = $entityTypeManager;
    $this->fileSystem = $fileSystem;
  }

  /**
   * Create pdf thumbnail.
   *
   * @param \Drupal\Core\Entity\Entity $entity
   *   Entity.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityStorageException
   * @throws \ImagickException
   */
  public function createThumbnail(Entity $entity) {
    $fieldDefinitions = $entity->getFieldDefinitions();
    if ($fieldDefinitions) {
      foreach ($fieldDefinitions as $fieldDefinition) {
        if ($fieldDefinition->getType() == "file") {
          if (!empty($entity->get($fieldDefinition->getName())->getValue())) {
            foreach ($entity->get($fieldDefinition->getName())->getValue() as $fieldValue) {
              if (array_key_exists('target_id', $fieldValue)) {
                $tid = $fieldValue['target_id'];
                $fileEntity = $this->entityTypeManager->getStorage('file')
                  ->load($tid);
                if ($fileEntity->getMimeType() == 'application/pdf') {
                  $fileUri = $fileEntity->getFileUri();
                  $fileUriArray = explode('/', $fileUri);
                  $fileName = end($fileUriArray);
                  $sourcePath = $this->fileSystem->realpath($fileUri);
                  if ($sourcePath) {
                    $imageFileUri = $this->mediaPdfThumbnailImagickManager->generateImage($sourcePath,
                      $fileName . '.png');
                    $fileEntityId = $this->createThumbnailFileEntity($imageFileUri);
                    if ($fileEntityId) {
                      $entity->set('thumbnail', ['target_id' => $fileEntityId]);
                    }
                  }
                }
              }
            }
          }
        }
      }
    }
  }

  /**
   * Create file entity.
   *
   * @param string $fileUri
   *   File uri.
   *
   * @return int|null|string
   *   File entity id.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  protected function createThumbnailFileEntity($fileUri) {
    if ($fileUri) {
      $fileEntity = $this->entityTypeManager->getStorage('file')->create([
        'uri' => $fileUri,
        'status' => FILE_STATUS_PERMANENT,
      ]);
      $fileEntity->save();
      return $fileEntity->id();
    }
    return NULL;
  }

}
